#!/bin/sh

# 
# Configures a bridge as defined in /etc/sysconfig/network-scirpts/brcfg-<br name>
#
# the parameters are:
#  BRIDGE:      name of the bridge, should match <br name>
#  ONBOOT:      wether to start the bridge at boot time (!="no")
#  INTERFACES:  (ethernet) interfaces to bind to this bridge
#  STP:         wether to enable STP on this bridge (="yes")
#

PATH=/sbin:/usr/sbin:/bin:/usr/bin

cwd=`pwd`
cd /etc/sysconfig/network-scripts

config=$1

[ -z "$config" ] && {
    echo "usage: brcfg <bridge name>" >&2
    exit 1
}

[ -f "$config" ] || config=brcfg-$config
[ -f "$config" ] || {
    echo "usage: brcfg <bridge name>" >&2
    exit 3
}

[ -x "/usr/bin/brctl" ]  && brctl=/usr/bin/brctl
[ -x "/usr/sbin/brctl" ] && brctl=/usr/sbin/brctl
[ -x "/bin/brctl" ]      && brctl=/bin/brctl
[ -x "/sbin/brctl" ]     && brctl=/sbin/brctl
[ -z "$brctl" ] && {
    echo "Can not find brctl utility." >&2
    exit 5
}

[ -x "/usr/bin/ip" ]  && iptool=/usr/bin/ip
[ -x "/usr/sbin/ip" ] && iptool=/usr/sbin/ip
[ -x "/bin/ip" ]      && iptool=/bin/ip
[ -x "/sbin/ip" ]     && iptool=/sbin/ip
[ -z "$iptool" ] && {
    [ -x "/usr/bin/ifconfig" ]  && ifcfg=/usr/bin/ifconfig
    [ -x "/usr/sbin/ifconfig" ] && ifcfg=/usr/sbin/ifconfig
    [ -x "/bin/ifconfig" ]      && ifcfg=/bin/ifconfig
    [ -x "/sbin/ifconfig" ]     && ifcfg=/sbin/ifconfig
    [ -z "$ifcfg" ] && {
        echo "Can not find ip or ifconfig utility." >&2
        exit 5
    }
}

if [ $UID != 0 ]; then
    echo "Users cannot cofigurate this bridge." >&2
    exit 1
fi

. $config

if [ " $BRIDGE" = " " ] ; then
    echo "No bridge name/interface" >&2
    exit 6
fi

if [ " $2" = " boot" -a " ${ONBOOT}" = " no" ]; then
    exit 6
fi

# get configured bridges
bridges=`$brctl show         | \
         tail +2             | \
         grep "^[a-zA-Z0-9]" | \
         sed -e "s/^\([a-zA-Z0-9]*\).*/\1/"`

# check is the bridge already exists
havebridge=
for i in $bridges ; do
    if [ "$i" = "$BRIDGE" ]; then
        havebridge=1
    fi
done

# if not try to create it
if [ -z "$havebridge" ] ; then
    $brctl addbr $BRIDGE && havebridge=1
else
    if [ ! " $2" = " reconfigure" ] ; then
        echo "Bridge $BRIDGE already running, use:" >&2
        echo "    brcfg $BRIDGE reconfigure" >&2
        echo "to force reconfiguration." >&2
        exit 1
    fi
fi

if [ -z "$havebridge" ] ; then
    echo "Unable to create bridge $BRIDGE." >&2
    exit 1
fi

# no error, yet
rc=0

# get all interfaces of the bridge
interfaces=$(
    $brctl show | tail +2 | while read br j1 j2 dev ; do
        [ "$dev" ] && bri=$br || dev=$br
        [ "$bri" = "$BRIDGE" ] && echo $dev
    done
    )

# remove old interfaces
for i in $interfaces ; do
    case $INTERFACES in
        *$i*)
            :
        ;;
        *)
            $brctl delif $BRIDGE $i || rc=1
        ;;
    esac
done
if [ $rc = 1 ] ; then
    echo "Error while removing deprecated interfaces from $BRIDGE." >&2
    exit 1
fi

# bringes up an interface
if_up () {
    local int=$1 rc=0

    if [ "$iptool" ] ; then
        $iptool link set $int up
        
    elif [ "$ifconfig" ] ; then
        $ifconfig $int inet 0.0.0.0 up || rc=1
    else
        rc=1
    fi

    return $rc
}


# add new interfaces
for i in $INTERFACES ; do
    case $interfaces in
        *$i*)
            :
        ;;
        *)
            if_up $i && \
            $brctl addif $BRIDGE $i || \
            rc=1
        ;;
    esac
done
if [ $rc = 1 ] ; then
    echo "Error while adding interfaces to $BRIDGE." >&2
    exit 1
fi


# disable stp
if [ " $STP" != " yes" ]; then
    brctl stp $BRIDGE no || rc=1
else
    STP=yes
    brctl stp $BRIDGE yes || rc=1
fi
if [ $rc = 1 ] ; then
    echo "Error setting STP to $STP on $BRIDGE." >&2
    exit 1
fi

